var path = require('path'),
    _ = require('underscore'),
    options = JSON.parse(process.argv[2]);
    
var currentModule = path.basename(options.code.path, '.js'), 
    currentTest;

// make qunit api global, like it is in the browser
_.extend(global, require('../deps/qunit/qunit/qunit'));

/**
 * Require a resource.
 * @param {Object} res
 */
function load(res) {
    var requirePath = res.path.replace(/\.js$/, '');

    // test resource can define'namespace'to expose its exports as a named object
    if (res.namespace) {
        global[res.namespace] = require(requirePath);
    } else {
        _.extend(global, require(requirePath));
    }
}

// add paths to require
if (options.paths){
    require.paths.push.apply(require.paths, options.paths);
}

/**
 * Calculate coverage stats using bunker
 */
function calcCoverage() {
    
}

/**
 * Callback for each started test.
 * @param {Object} test
 */
QUnit.testStart = function(test) {
    // currentTest is undefined while first test is not done yet
    currentTest = test.name;
    // use last module name if no module name defined
    currentModule = test.module || currentModule;
};

/**
 * Callback for each assertion.
 * @param {Object} data
 */
QUnit.log = function(data) {
    data.test = currentTest;
    data.module = currentModule;
    
    process.send({
        event: 'assertionDone',
        data: data
    });
};

/**
 * Callback for one done test.
 * @param {Object} test
 */
QUnit.testDone = function(test) {
    // use last module name if no module name defined
    test.module = test.module || currentModule;
    
    process.send({
        event: 'testDone',
        data: test
    });
};

/**
 * Callback for all done tests in the file.
 * @param {Object} res
 */
QUnit.done = function done(res) {
    // XXX don't know why, but qunit fires a lot of done callbacks,
    // only the last one has correct data #wtf
    clearTimeout(done.timeout);
    done.timeout = setTimeout(function() {
        if (options.coverage) {
            res.coverage = calcCoverage();    
        }
        
        process.send({
            event: 'done',
            data: res
        });        
    }, 300); 
};


// require deps
options.deps.forEach(load);
// require code
load(options.code);
// require tests
options.tests.forEach(load);

QUnit.begin();
